print("import: 'snowflake'")
import snowflake

print("import: 'snowflake.connector'")
import snowflake.connector

print("import: 'snowflake.connector.nanoarrow_arrow_iterator'")
import snowflake.connector.nanoarrow_arrow_iterator

